"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.propTypes = exports.allowedProps = void 0;
var _FormPropTypes = require("@instructure/ui-form-field/lib/FormPropTypes.js");
var _I18nPropTypes = require("@instructure/ui-i18n/lib/I18nPropTypes.js");
var _propTypes = _interopRequireDefault(require("prop-types"));
var _controllable = require("@instructure/ui-prop-types/lib/controllable.js");
/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2015 - present Instructure, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

const hourMinuteValidator = function (props, propName, componentName, location) {
  const propValue = props[propName];
  if (typeof propValue === 'undefined' || propValue === '') return null;
  const hourMinuteRegex = /^\d{2}:\d{2}$/;
  if (typeof propValue === 'string' && !propValue.match(hourMinuteRegex)) {
    return new Error(`Invalid ${location} \`${propName}\` \`${propValue}\` supplied to \`${componentName}\`, expected ` + `a HH:MM formatted string.`);
  }
  return null;
};
const propTypes = exports.propTypes = {
  description: _propTypes.default.node.isRequired,
  dateRenderLabel: _propTypes.default.oneOfType([_propTypes.default.func, _propTypes.default.node]).isRequired,
  prevMonthLabel: _propTypes.default.string.isRequired,
  nextMonthLabel: _propTypes.default.string.isRequired,
  datePlaceholder: _propTypes.default.string,
  timePlaceholder: _propTypes.default.string,
  dateFormat: _propTypes.default.string,
  interaction: _propTypes.default.oneOf(['enabled', 'disabled', 'readonly']),
  timeRenderLabel: _propTypes.default.oneOfType([_propTypes.default.func, _propTypes.default.node]).isRequired,
  timeStep: _propTypes.default.oneOf([5, 10, 15, 20, 30, 60]),
  timeFormat: _propTypes.default.string,
  locale: _propTypes.default.string,
  timezone: _propTypes.default.string,
  invalidDateTimeMessage: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.func]).isRequired,
  showMessages: _propTypes.default.bool,
  messages: _propTypes.default.arrayOf(_FormPropTypes.FormPropTypes.message),
  messageFormat: _propTypes.default.string,
  layout: _propTypes.default.oneOf(['stacked', 'columns', 'inline']),
  rowSpacing: _propTypes.default.oneOf(['none', 'small', 'medium', 'large']),
  colSpacing: _propTypes.default.oneOf(['none', 'small', 'medium', 'large']),
  value: (0, _controllable.controllable)(_I18nPropTypes.I18nPropTypes.iso8601, 'onChange'),
  defaultValue: _I18nPropTypes.I18nPropTypes.iso8601,
  renderWeekdayLabels: _propTypes.default.arrayOf(_propTypes.default.oneOfType([_propTypes.default.func, _propTypes.default.node])),
  isRequired: _propTypes.default.bool,
  onChange: _propTypes.default.func,
  dateInputRef: _propTypes.default.func,
  timeInputRef: _propTypes.default.func,
  onBlur: _propTypes.default.func,
  disabledDates: _propTypes.default.oneOfType([_propTypes.default.func, _propTypes.default.arrayOf(_propTypes.default.string)]),
  disabledDateTimeMessage: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.func]),
  allowNonStepInput: _propTypes.default.bool,
  reset: _propTypes.default.func,
  initialTimeForNewDate: hourMinuteValidator
};
const allowedProps = exports.allowedProps = ['description', 'dateRenderLabel', 'prevMonthLabel', 'nextMonthLabel', 'datePlaceholder', 'timePlaceholder', 'dateFormat', 'interaction', 'timeRenderLabel', 'timeStep', 'timeFormat', 'locale', 'timezone', 'invalidDateTimeMessage', 'showMessages', 'messages', 'messageFormat', 'layout', 'rowSpacing', 'colSpacing', 'value', 'defaultValue', 'renderWeekdayLabels', 'isRequired', 'onChange', 'dateInputRef', 'timeInputRef', 'onBlur', 'disabledDates', 'disabledDateTimeMessage', 'allowNonStepInput', 'reset'];